@@ -6,7 +6,7 @@
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -18,37 +18,40 @@ package brut.androlib.res.data;
 
 import brut.androlib.AndrolibException;
 import brut.androlib.err.UndefinedResObjectException;
-import java.util.*;
 import org.apache.commons.lang3.StringUtils;
 
-/**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- */
+import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
+import java.util.Map;
+import java.util.Set;
+
 public class ResResSpec {
     private final ResID mId;
     private final String mName;
     private final ResPackage mPackage;
     private final ResTypeSpec mType;
-    private final Map<ResConfigFlags, ResResource> mResources = new LinkedHashMap<ResConfigFlags, ResResource>();
+    private final Map<ResConfigFlags, ResResource> mResources = new LinkedHashMap<>();
 
     public ResResSpec(ResID id, String name, ResPackage pkg, ResTypeSpec type) {
         this.mId = id;
         String cleanName;
 
+        name = (("(name removed)".equals(name)) ? null : name);
+
         ResResSpec resResSpec = type.getResSpecUnsafe(name);
         if (resResSpec != null) {
-            cleanName = name + "_APKTOOL_DUPLICATENAME_" + id.toString();
+            cleanName = String.format("APKTOOL_DUPLICATE_%s_%s", type, id.toString());
         } else {
             cleanName = ((name == null || name.isEmpty()) ? ("APKTOOL_DUMMYVAL_" + id.toString()) : name);
         }
-        
+
         this.mName = cleanName;
         this.mPackage = pkg;
         this.mType = type;
     }
 
     public Set<ResResource> listResources() {
-        return new LinkedHashSet<ResResource>(mResources.values());
+        return new LinkedHashSet<>(mResources.values());
     }
 
     public ResResource getResource(ResType config) throws AndrolibException {
@@ -63,14 +66,6 @@ public class ResResSpec {
         return res;
     }
 
-    public boolean hasResource(ResType config) {
-        return hasResource(config.getFlags());
-    }
-
-    private boolean hasResource(ResConfigFlags flags) {
-        return mResources.containsKey(flags);
-    }
-
     public ResResource getDefaultResource() throws AndrolibException {
         return getResource(new ResConfigFlags());
     }
@@ -79,10 +74,6 @@ public class ResResSpec {
         return mResources.containsKey(new ResConfigFlags());
     }
 
-    public String getFullName() {
-        return getFullName(false, false);
-    }
-
     public String getFullName(ResPackage relativeToPackage, boolean excludeType) {
         return getFullName(getPackage().equals(relativeToPackage), excludeType);
     }
@@ -123,11 +114,6 @@ public class ResResSpec {
         }
     }
 
-    public void removeResource(ResResource res) throws AndrolibException {
-        ResConfigFlags flags = res.getConfig().getFlags();
-        mResources.remove(flags);
-    }
-
     @Override
     public String toString() {
         return mId.toString() + " " + mType.toString() + "/" + mName;
