@@ -6,7 +6,7 @@
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -20,21 +20,14 @@ import brut.androlib.AndrolibException;
 import brut.androlib.err.UndefinedResObjectException;
 import java.util.*;
 
-/**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- */
 public class ResType {
     private final ResConfigFlags mFlags;
-    private final Map<ResResSpec, ResResource> mResources = new LinkedHashMap<ResResSpec, ResResource>();
+    private final Map<ResResSpec, ResResource> mResources = new LinkedHashMap<>();
 
     public ResType(ResConfigFlags flags) {
         this.mFlags = flags;
     }
 
-    public Set<ResResource> listResources() {
-        return new LinkedHashSet<ResResource>(mResources.values());
-    }
-
     public ResResource getResource(ResResSpec spec) throws AndrolibException {
         ResResource res = mResources.get(spec);
         if (res == null) {
@@ -43,10 +36,6 @@ public class ResType {
         return res;
     }
 
-    public Set<ResResSpec> listResSpecs() {
-        return mResources.keySet();
-    }
-
     public ResConfigFlags getFlags() {
         return mFlags;
     }
@@ -55,11 +44,6 @@ public class ResType {
         addResource(res, false);
     }
 
-    public void removeResource(ResResource res) throws AndrolibException {
-        ResResSpec spec = res.getResSpec();
-        mResources.remove(spec);
-    }
-
     public void addResource(ResResource res, boolean overwrite) throws AndrolibException {
         ResResSpec spec = res.getResSpec();
         if (mResources.put(spec, res) != null && !overwrite) {
