@@ -6,7 +6,7 @@
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -16,22 +16,17 @@
  */
 package brut.androlib.res.data.value;
 
-import brut.androlib.AndrolibException;
 import brut.androlib.res.data.ResResource;
 import brut.androlib.res.xml.ResValuesXmlSerializable;
-import java.io.IOException;
 import org.xmlpull.v1.XmlSerializer;
 
-/**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- */
+import java.io.IOException;
+
 public class ResIdValue extends ResValue implements ResValuesXmlSerializable {
     @Override
-    public void serializeToResValuesXml(XmlSerializer serializer,
-                                        ResResource res) throws IOException, AndrolibException {
+    public void serializeToResValuesXml(XmlSerializer serializer, ResResource res) throws IOException {
         serializer.startTag(null, "item");
-        serializer
-                .attribute(null, "type", res.getResSpec().getType().getName());
+        serializer.attribute(null, "type", res.getResSpec().getType().getName());
         serializer.attribute(null, "name", res.getResSpec().getName());
         serializer.endTag(null, "item");
     }
