@@ -6,7 +6,7 @@
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -23,28 +23,24 @@ import brut.androlib.res.data.ResID;
 import brut.androlib.res.xml.ResXmlEncoders;
 import brut.util.ExtDataInput;
 import com.google.common.io.LittleEndianDataInputStream;
+import org.xmlpull.v1.XmlPullParserException;
 import java.io.DataInput;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.Reader;
 import java.util.logging.Level;
 import java.util.logging.Logger;
-import org.xmlpull.v1.XmlPullParserException;
 
 /**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- * @author Dmitry Skiba
+ * Binary xml files parser.
  *
- *         Binary xml files parser.
- *
- *         Parser has only two states: (1) Operational state, which parser
- *         obtains after first successful call to next() and retains until
- *         open(), close(), or failed call to next(). (2) Closed state, which
- *         parser obtains after open(), close(), or failed call to next(). In
- *         this state methods return invalid values or throw exceptions.
- *
- *         TODO: * check all methods in closed state
+ * Parser has only two states: (1) Operational state, which parser
+ * obtains after first successful call to next() and retains until
+ * open(), close(), or failed call to next(). (2) Closed state, which
+ * parser obtains after open(), close(), or failed call to next(). In
+ * this state methods return invalid values or throw exceptions.
  *
+ * TODO: * check all methods in closed state
  */
 public class AXmlResourceParser implements XmlResourceParser {
 
@@ -214,18 +210,18 @@ public class AXmlResourceParser implements XmlResourceParser {
     }
 
     @Override
-    public int getNamespaceCount(int depth) throws XmlPullParserException {
+    public int getNamespaceCount(int depth) {
         return m_namespaces.getAccumulatedCount(depth);
     }
 
     @Override
-    public String getNamespacePrefix(int pos) throws XmlPullParserException {
+    public String getNamespacePrefix(int pos) {
         int prefix = m_namespaces.getPrefix(pos);
         return m_strings.getString(prefix);
     }
 
     @Override
-    public String getNamespaceUri(int pos) throws XmlPullParserException {
+    public String getNamespaceUri(int pos) {
         int uri = m_namespaces.getUri(pos);
         return m_strings.getString(uri);
     }
@@ -340,16 +336,26 @@ public class AXmlResourceParser implements XmlResourceParser {
         }
 
         String value = m_strings.getString(name);
+        String namespace = getAttributeNamespace(index);
 
-        // some attributes will return "", we must rely on the resource_id and refer to the frameworks
-        // to match the resource id to the name. ex: 0x101021C = versionName
-        if (value.length() == 0 || android_ns.equals(getAttributeNamespace(index))) {
+        // If attribute name is lacking or a private namespace emerges,
+        // retrieve the exact attribute name by its id.
+        if (value == null || value.length() == 0) {
             try {
-                int resourceId = getAttributeNameResource(index);
-                if (resourceId != 0) {
-                    value = mAttrDecoder.decodeManifestAttr(getAttributeNameResource(index));
+                value = mAttrDecoder.decodeManifestAttr(getAttributeNameResource(index));
+                if (value == null) {
+                    value = "";
                 }
-            } catch (AndrolibException | NullPointerException e) { }
+            } catch (AndrolibException e) {
+                value = "";
+            }
+        } else if (! namespace.equals(android_ns)) {
+            try {
+                String obfuscatedName = mAttrDecoder.decodeManifestAttr(getAttributeNameResource(index));
+                if (! (obfuscatedName == null || obfuscatedName.equals(value))) {
+                    value = obfuscatedName;
+                }
+            } catch (AndrolibException ignored) {}
         }
         return value;
     }
@@ -385,11 +391,27 @@ public class AXmlResourceParser implements XmlResourceParser {
 
         if (mAttrDecoder != null) {
             try {
+                String value = valueRaw == -1 ? null : ResXmlEncoders.escapeXmlChars(m_strings.getString(valueRaw));
+                String obfuscatedValue = mAttrDecoder.decodeManifestAttr(valueData);
+
+                if (! (value == null || obfuscatedValue == null)) {
+                    int slashPos = value.lastIndexOf("/");
+
+                    if (slashPos != -1) {
+                        // Handle a value with a format of "@yyy/xxx"
+                        String dir = value.substring(0, slashPos);
+                        value = dir + "/"+ obfuscatedValue;
+                    } else if (! value.equals(obfuscatedValue)) {
+                        value = obfuscatedValue;
+                    }
+                }
+
                 return mAttrDecoder.decode(
-                        valueType,
-                        valueData,
-                        valueRaw == -1 ? null : ResXmlEncoders.escapeXmlChars(m_strings.getString(valueRaw)),
-                        getAttributeNameResource(index));
+                    valueType,
+                    valueData,
+                    value,
+                    getAttributeNameResource(index)
+                );
             } catch (AndrolibException ex) {
                 setFirstError(ex);
                 LOGGER.log(Level.WARNING, String.format("Could not decode attr value, using undecoded value "
@@ -521,8 +543,7 @@ public class AXmlResourceParser implements XmlResourceParser {
 
     // ///////////////////////////////// dummies
     @Override
-    public void setInput(InputStream stream, String inputEncoding)
-            throws XmlPullParserException {
+    public void setInput(InputStream stream, String inputEncoding) {
         open(stream);
     }
 
@@ -542,12 +563,12 @@ public class AXmlResourceParser implements XmlResourceParser {
     }
 
     @Override
-    public boolean isEmptyElementTag() throws XmlPullParserException {
+    public boolean isEmptyElementTag() {
         return false;
     }
 
     @Override
-    public boolean isWhitespace() throws XmlPullParserException {
+    public boolean isWhitespace() {
         return false;
     }
 
@@ -606,17 +627,12 @@ public class AXmlResourceParser implements XmlResourceParser {
             m_data = new int[32];
         }
 
-        public final void reset() {
+        public void reset() {
             m_dataLength = 0;
-            m_count = 0;
             m_depth = 0;
         }
 
-        public final int getTotalCount() {
-            return m_count;
-        }
-
-        public final int getCurrentCount() {
+        public int getCurrentCount() {
             if (m_dataLength == 0) {
                 return 0;
             }
@@ -624,7 +640,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             return m_data[offset];
         }
 
-        public final int getAccumulatedCount(int depth) {
+        public int getAccumulatedCount(int depth) {
             if (m_dataLength == 0 || depth < 0) {
                 return 0;
             }
@@ -641,7 +657,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             return accumulatedCount;
         }
 
-        public final void push(int prefix, int uri) {
+        public void push(int prefix, int uri) {
             if (m_depth == 0) {
                 increaseDepth();
             }
@@ -653,38 +669,9 @@ public class AXmlResourceParser implements XmlResourceParser {
             m_data[offset + 1] = uri;
             m_data[offset + 2] = count + 1;
             m_dataLength += 2;
-            m_count += 1;
         }
 
-        public final boolean pop(int prefix, int uri) {
-            if (m_dataLength == 0) {
-                return false;
-            }
-            int offset = m_dataLength - 1;
-            int count = m_data[offset];
-            for (int i = 0, o = offset - 2; i != count; ++i, o -= 2) {
-                if (m_data[o] != prefix || m_data[o + 1] != uri) {
-                    continue;
-                }
-                count -= 1;
-                if (i == 0) {
-                    m_data[o] = count;
-                    o -= (1 + count * 2);
-                    m_data[o] = count;
-                } else {
-                    m_data[offset] = count;
-                    offset -= (1 + 2 + count * 2);
-                    m_data[offset] = count;
-                    System.arraycopy(m_data, o + 2, m_data, o, m_dataLength - o);
-                }
-                m_dataLength -= 2;
-                m_count -= 1;
-                return true;
-            }
-            return false;
-        }
-
-        public final boolean pop() {
+        public boolean pop() {
             if (m_dataLength == 0) {
                 return false;
             }
@@ -699,31 +686,26 @@ public class AXmlResourceParser implements XmlResourceParser {
             offset -= (1 + count * 2);
             m_data[offset] = count;
             m_dataLength -= 2;
-            m_count -= 1;
             return true;
         }
 
-        public final int getPrefix(int index) {
+        public int getPrefix(int index) {
             return get(index, true);
         }
 
-        public final int getUri(int index) {
+        public int getUri(int index) {
             return get(index, false);
         }
 
-        public final int findPrefix(int uri) {
+        public int findPrefix(int uri) {
             return find(uri, false);
         }
 
-        public final int findUri(int prefix) {
-            return find(prefix, true);
-        }
-
-        public final int getDepth() {
+        public int getDepth() {
             return m_depth;
         }
 
-        public final void increaseDepth() {
+        public void increaseDepth() {
             ensureDataCapacity(2);
             int offset = m_dataLength;
             m_data[offset] = 0;
@@ -732,7 +714,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             m_depth += 1;
         }
 
-        public final void decreaseDepth() {
+        public void decreaseDepth() {
             if (m_dataLength == 0) {
                 return;
             }
@@ -742,7 +724,6 @@ public class AXmlResourceParser implements XmlResourceParser {
                 return;
             }
             m_dataLength -= 2 + count * 2;
-            m_count -= count;
             m_depth -= 1;
         }
 
@@ -757,7 +738,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             m_data = newData;
         }
 
-        private final int find(int prefixOrUri, boolean prefix) {
+        private int find(int prefixOrUri, boolean prefix) {
             if (m_dataLength == 0) {
                 return -1;
             }
@@ -781,7 +762,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             return -1;
         }
 
-        private final int get(int index, boolean prefix) {
+        private int get(int index, boolean prefix) {
             if (m_dataLength == 0 || index < 0) {
                 return -1;
             }
@@ -804,15 +785,10 @@ public class AXmlResourceParser implements XmlResourceParser {
 
         private int[] m_data;
         private int m_dataLength;
-        private int m_count;
         private int m_depth;
     }
 
-    final StringBlock getStrings() {
-        return m_strings;
-    }
-
-    private final int getAttributeOffset(int index) {
+    private int getAttributeOffset(int index) {
         if (m_event != START_TAG) {
             throw new IndexOutOfBoundsException("Current event is not START_TAG.");
         }
@@ -823,7 +799,7 @@ public class AXmlResourceParser implements XmlResourceParser {
         return offset;
     }
 
-    private final int findAttribute(String namespace, String attribute) {
+    private int findAttribute(String namespace, String attribute) {
         if (m_strings == null || attribute == null) {
             return -1;
         }
@@ -841,7 +817,7 @@ public class AXmlResourceParser implements XmlResourceParser {
         return -1;
     }
 
-    private final void resetEventInfo() {
+    private void resetEventInfo() {
         m_event = -1;
         m_lineNumber = -1;
         m_name = -1;
@@ -852,14 +828,14 @@ public class AXmlResourceParser implements XmlResourceParser {
         m_styleAttribute = -1;
     }
 
-    private final void doNext() throws IOException {
+    private void doNext() throws IOException {
         // Delayed initialization.
         if (m_strings == null) {
             m_reader.skipCheckInt(CHUNK_AXML_FILE, CHUNK_AXML_FILE_BROKEN);
 
-			/*
-			 * chunkSize
-			 */
+            /*
+             * chunkSize
+             */
             m_reader.skipInt();
             m_strings = StringBlock.read(m_reader);
             m_namespaces.increaseDepth();
@@ -913,9 +889,9 @@ public class AXmlResourceParser implements XmlResourceParser {
             }
 
             // Common header.
-			/* chunkSize */m_reader.skipInt();
+            /* chunkSize */m_reader.skipInt();
             int lineNumber = m_reader.readInt();
-			/* 0xFFFFFFFF */m_reader.skipInt();
+            /* 0xFFFFFFFF */m_reader.skipInt();
 
             if (chunkType == CHUNK_XML_START_NAMESPACE || chunkType == CHUNK_XML_END_NAMESPACE) {
                 if (chunkType == CHUNK_XML_START_NAMESPACE) {
@@ -923,8 +899,8 @@ public class AXmlResourceParser implements XmlResourceParser {
                     int uri = m_reader.readInt();
                     m_namespaces.push(prefix, uri);
                 } else {
-					/* prefix */m_reader.skipInt();
-					/* uri */m_reader.skipInt();
+                    /* prefix */m_reader.skipInt();
+                    /* uri */m_reader.skipInt();
                     m_namespaces.pop();
                 }
                 continue;
@@ -935,7 +911,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             if (chunkType == CHUNK_XML_START_TAG) {
                 m_namespaceUri = m_reader.readInt();
                 m_name = m_reader.readInt();
-				/* flags? */m_reader.skipInt();
+                /* flags? */m_reader.skipInt();
                 int attributeCount = m_reader.readInt();
                 m_idAttribute = (attributeCount >>> 16) - 1;
                 attributeCount &= 0xFFFF;
@@ -962,8 +938,8 @@ public class AXmlResourceParser implements XmlResourceParser {
 
             if (chunkType == CHUNK_XML_TEXT) {
                 m_name = m_reader.readInt();
-				/* ? */m_reader.skipInt();
-				/* ? */m_reader.skipInt();
+                /* ? */m_reader.skipInt();
+                /* ? */m_reader.skipInt();
                 m_event = TEXT;
                 break;
             }
@@ -977,10 +953,10 @@ public class AXmlResourceParser implements XmlResourceParser {
     }
 
     // ///////////////////////////////// data
-	/*
-	 * All values are essentially indices, e.g. m_name is an index of name in
-	 * m_strings.
-	 */
+    /*
+     * All values are essentially indices, e.g. m_name is an index of name in
+     * m_strings.
+     */
     private ExtDataInput m_reader;
     private ResAttrDecoder mAttrDecoder;
     private AndrolibException mFirstError;
@@ -988,7 +964,7 @@ public class AXmlResourceParser implements XmlResourceParser {
     private boolean m_operational = false;
     private StringBlock m_strings;
     private int[] m_resourceIDs;
-    private NamespaceStack m_namespaces = new NamespaceStack();
+    private final NamespaceStack m_namespaces = new NamespaceStack();
     private final String android_ns = "http://schemas.android.com/apk/res/android";
     private boolean m_decreaseDepth;
     private int m_event;
