@@ -6,7 +6,7 @@
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -20,10 +20,7 @@ import brut.androlib.AndrolibException;
 import java.io.InputStream;
 import java.io.OutputStream;
 
-/**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- */
 public interface ResStreamDecoder {
-    public void decode(InputStream in, OutputStream out)
+    void decode(InputStream in, OutputStream out)
             throws AndrolibException;
 }
