@@ -6,7 +6,7 @@
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -18,11 +18,8 @@ package brut.androlib.res.xml;
 
 import brut.androlib.AndrolibException;
 
-/**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- */
 public interface ResXmlEncodable {
-    public String encodeAsResXmlAttr() throws AndrolibException;
+    String encodeAsResXmlAttr() throws AndrolibException;
 
-    public String encodeAsResXmlValue() throws AndrolibException;
+    String encodeAsResXmlValue() throws AndrolibException;
 }
