@@ -6,7 +6,7 @@
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -21,25 +21,23 @@ import brut.androlib.ApkDecoder;
 import brut.androlib.BaseTest;
 import brut.androlib.TestUtils;
 import brut.androlib.meta.MetaInfo;
-import brut.directory.ExtFile;
 import brut.common.BrutException;
+import brut.directory.ExtFile;
 import brut.util.OS;
-
-import java.awt.image.BufferedImage;
-import java.io.*;
-import java.util.Map;
-
 import brut.util.OSDetection;
-import org.junit.*;
-
-import static org.junit.Assert.*;
-import static org.junit.Assume.*;
+import org.junit.AfterClass;
+import org.junit.BeforeClass;
+import org.junit.Test;
 
 import javax.imageio.ImageIO;
+import java.awt.image.BufferedImage;
+import java.io.File;
+import java.io.IOException;
+import java.util.Map;
+
+import static org.junit.Assert.*;
+import static org.junit.Assume.assumeTrue;
 
-/**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- */
 public class BuildAndDecodeTest extends BaseTest {
 
     @BeforeClass
@@ -178,7 +176,7 @@ public class BuildAndDecodeTest extends BaseTest {
     public void storedMp3FilesAreNotCompressedTest() throws BrutException {
         ExtFile extFile = new ExtFile(sTmpDir, "testapp.apk");
         Integer built = extFile.getDirectory().getCompressionLevel("res/raw/rain.mp3");
-        assertEquals(new Integer(0), built);
+        assertEquals(Integer.valueOf(0), built);
     }
 
     @Test
@@ -497,7 +495,7 @@ public class BuildAndDecodeTest extends BaseTest {
         MetaInfo metaInfo = new Androlib().readMetaFile(sTestNewDir);
 
         for (String item : metaInfo.doNotCompress) {
-            assertFalse(item.equals("jpg"));
+            assertNotEquals("jpg", item);
         }
     }
 
